#!/usr/bin/env python
# -*- coding: utf-8 -*-

# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Rel 1: MareroQ 01/05/2023
#        Initial release - 
#        Plugin based on plugin Tin Tran character-layers-from-text.py) https://gimplearn.net/viewtopic.php?f=3&t=2606
#        Randomly changes angle, height, and letter spacing ("fake handwriting").
#        Creates a layer group as a new image.(with individual letters on separate layers).
# ------------

import math
import string
from gimpfu import *
from array import array
import sys
import random
import codecs

def python_fu_text_layer_wobbly(image, textlayer, text,selectionCase,font,font_size,color,justify, angle, shift_x, shift_y):
	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	
	gimp.set_foreground(color)
	
	textlayer = pdb.gimp_text_fontname(image, None, 0, 0, text, 0, 1, font_size, PIXELS, font)

	text=text.lstrip(codecs.BOM_UTF8.decode("utf8", "strict"))
	if selectionCase==1:
		text=text.lower()
	elif selectionCase==2:
		text=text.upper()
	elif selectionCase==3:
		text=text.capitalize()		
	elif selectionCase==4:
		text=text.title()
	elif selectionCase==5:
		text=text.swapcase
		
	pdb.gimp_displays_flush()	
	font = pdb.gimp_text_layer_get_font(textlayer)
	font_size,unit = pdb.gimp_text_layer_get_font_size(textlayer)

	new_image = pdb.gimp_image_new(image.width,image.height,RGB)
	new_display = pdb.gimp_display_new(new_image) #creates a display for our new image
	new_layer_group = pdb.gimp_layer_group_new(new_image)
	pdb.gimp_image_insert_layer(new_image,new_layer_group,None,0)
	#pdb.gimp_item_set_name(new_layer_group,"Character Layers from Text")

	offset_x,offset_y = pdb.gimp_drawable_offsets(textlayer)
 
	lines = text.split("\n")
	starting_y = offset_y
	for line in lines:
		textlayers = []
		totalwidth = 0
		for i in range(0,len(line)):
			char = line[i]
			if ((char == " ") and (i > 0) and (i < len(line)-1)): #if space and it's between 2 chararacters we treat it special
				before = line[i-1]
				space = line[i]
				after = line[i+1]
				special = before + space + after
				special_layer = pdb.gimp_text_fontname(new_image,None,0,0,special,0,TRUE,font_size,unit,font)
				before_layer = pdb.gimp_text_fontname(new_image,None,0,0,before,0,TRUE,font_size,unit,font)
				after_layer = pdb.gimp_text_fontname(new_image,None,0,0,after,0,TRUE,font_size,unit,font)
				layer_height = special_layer.height
				layer_width = special_layer.width - before_layer.width - after_layer.width
				text_layer = pdb.gimp_layer_new(new_image,layer_width,layer_height,RGBA_IMAGE,"Space",100,LAYER_MODE_NORMAL)
				pdb.gimp_image_insert_layer(new_image,text_layer,new_layer_group,0)
				pdb.gimp_image_remove_layer(new_image,before_layer)
				pdb.gimp_image_remove_layer(new_image,after_layer)
				pdb.gimp_image_remove_layer(new_image,special_layer)
			else:
				text_layer = pdb.gimp_text_fontname(new_image,None,0,0,char,0,TRUE,font_size,unit,font)
				pdb.gimp_image_reorder_item(new_image,text_layer,new_layer_group,0) #puts it inside our layer group for organization
			totalwidth += text_layer.width #increment total width to calculate for placement and justify.
			textlayers.append(text_layer)
		if (justify == 0): #left justify
			starting_x = offset_x
		elif (justify == 1): #centered
			starting_x = offset_x + (textlayer.width/2) - totalwidth/2
		elif (justify == 2): #right justify
			starting_x = offset_x + textlayer.width - totalwidth
		for text_layer in textlayers: #loop through and move it into its placement
			pdb.gimp_layer_set_offsets(text_layer,starting_x,starting_y)
			starting_x += text_layer.width
		starting_y += text_layer.height #increment it for next round/line

		for text_layer in textlayers:

			if angle !=0:
				a = random.randrange(-angle,angle)
			else:
				a=0
			w, h, ascent, descent = pdb.gimp_text_get_extents_fontname(text_layer, font_size, 0, font)
			pdb.gimp_item_transform_rotate(text_layer, a*0.01745329252 ,1, w/2, h/2)	
		for text_layer in textlayers:
			a,b = pdb.gimp_drawable_offsets(text_layer)

			if shift_x !=0:
				x = random.randrange(-shift_x,shift_x)
 			else:
				x=0
				
			if shift_y !=0:				
				y = random.randrange(-shift_y,shift_y)
 			else:
				y=0
			pdb.gimp_layer_set_offsets(text_layer,a+x,b+y)	

	pdb.gimp_image_resize_to_layers(new_image)
	
	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()


register(
	"python_fu_text_layer_wobbly",
	"Random deformation of characters in the text layer",
	"Random deformation of characters in the text layer",
	"MareroQ",
	"GPL",
	"2023/05/01",
	"Random wobble text layer...",             # Menu name
	"RGB*, GRAY*",
	[

	(PF_IMAGE, "image", "Input image", None),
	(PF_DRAWABLE, "drawable", "Input drawable", None),
	(PF_TEXT, "p04", "TEXT:", "bar"),
	(PF_OPTION,"selectionCase",   "Change  Case?", 0, ["None","lowercase", "UPPERCASE", "Capitalize","Title", "Swapcase"]),
	(PF_FONT,     "font", "Font:", "Arial"),
	(PF_SPINNER,  "font_size", "Font size (px):", 100, (10, 5000, 1)),
	(PF_COLOR,    "color","Font Color: ", (255,255,255)),	
	(PF_OPTION,"justify",   "Justified:", 0, ["Left Justified","Centered","Right Justified"]),
	(PF_SPINNER, "angle", "Max. random angle", 0, (0, 45, 1)),
	(PF_SPINNER, "shift_x" ,"Max. random shift X", 0, (0, 20, 1)),
	(PF_SPINNER, "shift_y", "Max. random shift Y", 0, (0, 20, 1))
	],
	[],
	python_fu_text_layer_wobbly, 
	menu="<Image>/Filters/Text/")   # Menu path

main()

